@@ -1,21 +1,22 @@
 from __future__ import annotations
 from functools import wraps as functools_wraps
 from typing import Callable, TYPE_CHECKING, TypeVar
+from typing_extensions import _AnnotatedAlias
 import inspect
 from enum import Enum
 import warnings
-from qtpy.QtCore import Qt
+
 from magicgui import magicgui
 from magicgui.signature import MagicParameter
 from magicgui.widgets import FunctionGui, FileEdit, EmptyWidget
-
-from magicclass.signature import MagicMethodSignature
+from magicgui.widgets._bases import ValueWidget
 
 from .macro import Macro, Expr, Head, Symbol, symbol
-from .utils import iter_members, n_parameters, extract_tooltip, raise_error_in_msgbox, id_wrapper, screen_center
+from .utils import get_signature, iter_members, n_parameters, extract_tooltip, raise_error_in_msgbox, identity_wrapper, screen_center
 from .widgets import Separator, MacroEdit
-from .mgui_ext import FunctionGuiPlus
-from .field import MagicField
+from .mgui_ext import FunctionGuiPlus, PushButtonPlus
+from .fields import MagicField
+from .signature import MagicMethodSignature
 from .wrappers import upgrade_signature
 
 if TYPE_CHECKING:
@@ -66,10 +67,16 @@ class BaseGui:
                 continue
             macro.append(expr.format({sym: self_symbol}))
         
-        for name, attr in filter(lambda x: not x[0].startswith("__"), self.__dict__.items()):
+        for name, clsattr in iter_members(self.__class__):
             # Collect all the macro from child magic-classes recursively
+            if not isinstance(clsattr, (BaseGui, type, MagicField)):
+                continue
+            
+            attr = getattr(self, name)
+            
             if not isinstance(attr, BaseGui):
                 continue
+            
             macro += attr._collect_macro(self_symbol, Symbol(name))
                 
         return macro
@@ -187,7 +194,6 @@ class BaseGui:
                     f = _wraps(template, reference=parent_method)(f)
                     f.__wrapped__ = parent_method
                     f.__name__ = parent_method.__name__
-                    f.__qualname__ = parent_method.__qualname__
                     return f
                     
             # If method is defined as a member of class X, __qualname__ will be X.<method name>.
@@ -209,7 +215,7 @@ class BaseGui:
                 clsname, funcname = parent_method.__qualname__.split(".")
                 childmethod = _wrap_func(_childmethod)
             
-            # To avoid two buttons with same bound function being showed up
+            # To avoid two buttons with same bound function being shown up
             upgrade_signature(parent_method, caller_options={"visible": False})
             
             if hasattr(cls, funcname):
@@ -257,7 +263,7 @@ class BaseGui:
         if self._error_mode == ErrorMode.msgbox:
             wrapper = raise_error_in_msgbox
         elif self._error_mode == ErrorMode.stderr:
-            wrapper = id_wrapper
+            wrapper = identity_wrapper
         else:
             raise ValueError(self._error_mode)
             
@@ -276,28 +282,38 @@ class BaseGui:
         n_empty = len([widget for widget in fgui if isinstance(widget, EmptyWidget)])
         nparams = n_parameters(func) - n_empty
         
-        record = True
+        # This block enables instance methods in "bind" method of ValueWidget.
+        all_params = []
+        for param in func.__signature__.parameters.values():
+            if isinstance(param.annotation, _AnnotatedAlias):
+                param = MagicParameter.from_parameter(param)
+            if isinstance(param, MagicParameter):
+                bound_value = param.options.get("bind", None)
+                
+                # If bound method is a class method, use self.method(widget) as the value.
+                # TODO: n_parameters(bound_value) == 2 is not elegant
+                if isinstance(bound_value, Callable) and n_parameters(bound_value) == 2:
+                    param.options["bind"] = _method_as_getter(self, bound_value)
+                
+                # If a MagicFiled is bound, bind the value of the connected widget.
+                elif isinstance(bound_value, MagicField):
+                    param.options["bind"] = _field_as_getter(self, bound_value)
+                
+                # If a value widget is bound, bind the value.
+                elif isinstance(bound_value, ValueWidget):
+                    param.options["bind"] = _one_more_arg(bound_value.get_value)
+            
+            all_params.append(param)
         
-        if hasattr(obj, "__signature__"):
-            # This block enables instance methods in "bind" method of ValueWidget.
-            for param in obj.__signature__.parameters.values():
-                if isinstance(param, MagicParameter):
-                    bound_value = param.options.get("bind", None)
-                    # TODO: n_parameters(bound_value) == 2 is not elegant
-                    if callable(bound_value) and n_parameters(bound_value) == 2:
-                        clsname, _ = bound_value.__qualname__.split(".")
-                        if clsname != self.__class__.__name__:
-                            self.__class__.wraps(bound_value)
-                        param.options["bind"] = getattr(self, bound_value.__name__)
-                        
-            func.__signature__ = func.__signature__.replace(
-                parameters=list(obj.__signature__.parameters.values())[1:]
-                )
-
-            # Check additional options
-            if isinstance(obj.__signature__, MagicMethodSignature):
-                record = obj.__signature__.additional_options.get("record", True)
-                 
+        func.__signature__ = func.__signature__.replace(
+            parameters=all_params
+            )
+        
+        obj_sig = get_signature(obj)
+        if isinstance(func.__signature__, MagicMethodSignature):
+            # TODO: I don't know the reason why "additional_options" is lost. 
+            func.__signature__.additional_options = getattr(obj_sig, "additional_options", {})
+            
         if nparams == 0:
             # We don't want a dialog with a single widget "Run" to show up.
             def run_function():
@@ -305,10 +321,10 @@ class BaseGui:
                 # "compiled" otherwise function wrappings are not ready!
                 mgui = _build_mgui(widget, func)
                 mgui.native.setParent(self.native, mgui.native.windowFlags())
-                if mgui.call_count == 0 and len(mgui.called._slots) == 0 and record:
+                if mgui.call_count == 0 and len(mgui.called._slots) == 0 and _need_record(func):
                     callback = _temporal_function_gui_callback(self, mgui, widget)
                     mgui.called.connect(callback)
-
+                
                 out = mgui()
                 
                 return out
@@ -318,7 +334,7 @@ class BaseGui:
             def run_function():
                 mgui = _build_mgui(widget, func)
                 mgui.native.setParent(self.native, mgui.native.windowFlags())
-                if mgui.call_count == 0 and len(mgui.called._slots) == 0 and record:
+                if mgui.call_count == 0 and len(mgui.called._slots) == 0 and _need_record(func):
                     callback = _temporal_function_gui_callback(self, mgui, widget)
                     mgui.called.connect(callback)
                 
@@ -329,6 +345,7 @@ class BaseGui:
                     start_path=str(fdialog.value),
                     filter=fdialog.filter,
                 )
+
                 if result:
                     fdialog.value = result
                     out = mgui(result)
@@ -342,6 +359,8 @@ class BaseGui:
                 if mgui.call_count == 0 and len(mgui.called._slots) == 0:
                     mgui.native.setParent(self.native, mgui.native.windowFlags())
                     mgui.native.move(screen_center() - mgui.native.rect().center())
+                    
+                    # deal with popup mode.
                     if self._popup_mode not in (PopUpMode.popup, PopUpMode.dock):
                         mgui.label = ""
                         mgui.name = f"mgui-{id(mgui._function)}" # to avoid name collision
@@ -381,6 +400,7 @@ class BaseGui:
                             
                             else:    
                                 from qtpy.QtWidgets import QDockWidget
+                                from qtpy.QtCore import Qt
                                 dock = QDockWidget(_get_widget_name(widget), self.native)
                                 dock.setWidget(mgui.native)
                                 parent_self.native.addDockWidget(
@@ -398,10 +418,10 @@ class BaseGui:
                             # If FunctioGui is docked, we should close QDockWidget.
                             mgui.called.connect(lambda: mgui.parent.hide())
                     
-                    if record:
+                    if _need_record(func):
                         callback = _temporal_function_gui_callback(self, mgui, widget)
                         mgui.called.connect(callback)
-                
+                                    
                 if self._popup_mode != PopUpMode.dock:
                     widget.mgui.show()
                 else:
@@ -426,7 +446,7 @@ def _get_widget_name(widget):
     # To escape reference
     return widget.name
     
-def _temporal_function_gui_callback(bgui: BaseGui, fgui: FunctionGuiPlus, widget):
+def _temporal_function_gui_callback(bgui: BaseGui, fgui: FunctionGuiPlus, widget: PushButtonPlus):
     if isinstance(fgui, FunctionGui):
         _function = fgui._function
     else:
@@ -473,6 +493,8 @@ def _temporal_function_gui_callback(bgui: BaseGui, fgui: FunctionGuiPlus, widget
                 line = Expr.parse_call(callback, (_gui, result, return_type), {})
                 bgui._recorded_macro.append(line)
         
+        return None
+    
     return _after_run
 
 def _copy_function(f):
@@ -481,6 +503,11 @@ def _copy_function(f):
         return f(self, *args, **kwargs)
     return out
 
+def _one_more_arg(f):
+    def out(e):
+        return f()
+    return out
+
 def _get_index(container, widget_or_name):
     if isinstance(widget_or_name, str):
         name = widget_or_name
@@ -490,7 +517,7 @@ def _get_index(container, widget_or_name):
         if widget.name == name:
             break
     else:
-        raise ValueError
+        raise ValueError(f"{widget_or_name} not found in {container}")
     return i
 
 def _search_wrapper(bgui: BaseGui, funcname: str, clsname: str) -> tuple[BaseGui, Callable]:
@@ -586,4 +613,47 @@ def _wraps(template: Callable | inspect.Signature,
             )
         return f
     return wrapper
-    
\ No newline at end of file
+
+def _method_as_getter(self, bound_value: Callable):
+    *clsnames, funcname = bound_value.__qualname__.split(".")
+    
+    def _func(w):
+        ins = self
+        while clsnames[0] != ins.__class__.__name__:
+            ins = getattr(ins, "__magicclass_parent__", None)
+            if ins is None:
+                raise ValueError(f"Method {bound_value.__qualname__} is invisible"
+                                 f"from magicclass {self.__class__.__qualname__}")
+        
+        for clsname in clsnames[1:]:
+            ins = getattr(ins, clsname)
+        return getattr(ins, funcname)(w)
+    return _func
+
+def _field_as_getter(self, bound_value: MagicField):
+    def _func(w):
+        namespace = bound_value.parent_class.__qualname__
+        clsnames = namespace.split(".")
+        ins = self
+        while type(ins).__name__ not in clsnames:
+            ins = getattr(ins, "__magicclass_parent__", None)
+            if ins is None:
+                raise ValueError(f"MagicField {namespace}.{bound_value.name} is invisible"
+                                 f"from magicclass {self.__class__.__qualname__}")
+        i = clsnames.index(type(ins).__name__)
+        for clsname in clsnames[i:]:
+            ins = getattr(ins, clsname, ins)
+            
+        _field_widget = bound_value.get_widget(ins)
+        if not hasattr(_field_widget, "value"):
+            raise TypeError(f"MagicField {bound_value.name} does not return ValueWidget "
+                            "thus cannot be used as a bound value.")
+        return bound_value.as_getter(ins)(w)
+    return _func
+
+def _need_record(func: Callable):
+    if isinstance(func.__signature__, MagicMethodSignature):
+        record = func.__signature__.additional_options.get("record", True)
+    else:
+        record = True
+    return record
\ No newline at end of file
