@@ -3,7 +3,11 @@ from functools import wraps as functools_wraps
 import inspect
 from enum import Enum
 from dataclasses import is_dataclass
+from typing import Any
 import warnings
+from typing_extensions import Annotated, _AnnotatedAlias
+
+from magicclass.fields import MagicField, MISSING
 
 from .class_gui import (
     ClassGui, 
@@ -49,6 +53,16 @@ _TYPE_MAP = {
     WidgetType.mainwindow: MainWindowClassGui,
 }
 
+def Bound(value: Any) -> _AnnotatedAlias:
+    """
+    Make Annotated type from a MagicField or a method.
+    """    
+    # It is better to annotate like Annotated[int, {...}] but some widgets does not
+    # support bind. Also, we must ensure that parameters annotated with "bind" creates
+    # EmptyWidget.
+    
+    return Annotated[Any, {"bind": value}]
+
 def magicclass(class_: type|None = None,
                *,
                layout: str = "vertical", 
@@ -122,6 +136,8 @@ def magicclass(class_: type|None = None,
         doc = cls.__doc__
         sig = inspect.signature(cls)
         annot = cls.__dict__.get("__annotations__", {})
+        mod = cls.__module__
+        qualname = cls.__qualname__
         
         oldclass = type(cls.__name__ + _BASE_CLASS_SUFFIX, (cls,), {})
         newclass = type(cls.__name__, (class_gui, oldclass), {})
@@ -132,6 +148,8 @@ def magicclass(class_: type|None = None,
         # concatenate annotations
         newclass.__annotations__ = class_gui.__annotations__.copy()
         newclass.__annotations__.update(annot)
+        newclass.__module__ = mod
+        newclass.__qualname__ = qualname
         
         @functools_wraps(oldclass.__init__)
         def __init__(self, *args, **kwargs):
