@@ -1,103 +1,16 @@
 from __future__ import annotations
 from typing import Callable
 import warnings
-from magicgui.events import Signal
 from magicgui.widgets import Container
 from magicgui.widgets._bases import ButtonWidget
-from qtpy.QtGui import QIcon
-from qtpy.QtWidgets import QMenu, QAction
+from qtpy.QtWidgets import QMenu
 
-from .field import MagicField
+from .fields import MagicField
 from .widgets import Separator
+from .mgui_ext import Action
 from ._base import BaseGui, PopUpMode, ErrorMode
 from .macro import Expr, Head, Symbol, symbol
-from .utils import iter_members, define_callback
-
-class Action:
-    """QAction encapsulated class with a similar API as magicgui Widget."""
-    changed = Signal(object)
-    def __init__(self, *args, name=None, text=None, gui_only=True, **kwargs):
-        self.native = QAction(*args, **kwargs)
-        self.mgui = None
-        self._icon_path = None
-        if text:
-            self.text = text
-        if name:
-            self.native.setObjectName(name)
-        self._callbacks = []
-        
-        self.native.triggered.connect(lambda: self.changed.emit(self.value))
-        
-    @property
-    def name(self) -> str:
-        return self.native.objectName()
-    
-    @name.setter
-    def name(self, value: str):
-        self.native.setObjectName(value)
-    
-    @property
-    def text(self) -> str:
-        return self.native.text()
-    
-    @text.setter
-    def text(self, value: str):
-        self.native.setText(value)
-    
-    @property
-    def tooltip(self) -> str:
-        return self.native.toolTip()
-    
-    @tooltip.setter
-    def tooltip(self, value: str):
-        self.native.setToolTip(value)
-    
-    @property
-    def enabled(self):
-        return self.native.isEnabled()
-    
-    @enabled.setter
-    def enabled(self, value: bool):
-        self.native.setEnabled(value)
-    
-    @property
-    def value(self):
-        return self.native.isChecked()
-    
-    @value.setter
-    def value(self, checked: bool):
-        self.native.setChecked(checked)
-    
-    @property
-    def visible(self):
-        return self.native.isVisible()
-    
-    @visible.setter
-    def visible(self, value: bool):
-        self.native.setVisible(value)
-    
-    @property
-    def icon_path(self):
-        return self._icon_path
-    
-    @icon_path.setter
-    def icon_path(self, path:str):
-        icon = QIcon(path)
-        self.native.setIcon(icon)
-    
-    def from_options(self, options: dict[str]|Callable):
-        if callable(options):
-            try:
-                options = options.__signature__.caller_options
-            except AttributeError:
-                return None
-                
-        for k, v in options.items():
-            v = options.get(k, None)
-            if v is not None:
-                setattr(self, k, v)
-        return None
-
+from .utils import iter_members, define_callback, InvalidMagicClassError
 
 class MenuGuiBase(BaseGui):
     _component_class = Action
@@ -143,8 +56,8 @@ class MenuGuiBase(BaseGui):
             except (AttributeError, KeyError):
                 pass
             
+        fld.name = fld.name or name.replace("_", " ")
         widget = fld.to_widget()
-        widget.name = widget.name or name
             
         if isinstance(widget, ButtonWidget):
             # If the field has callbacks, connect it to the newly generated widget.
@@ -154,9 +67,9 @@ class MenuGuiBase(BaseGui):
                 widget.changed.connect(define_callback(self, callback))
                      
             # By default, set value function will be connected to the widget.
-            widget.changed.connect(_value_widget_callback(self, widget, name))
+            if fld.record:
+                widget.changed.connect(_value_widget_callback(self, widget, name))
                 
-        setattr(self, name, widget)
         return widget
     
     def _convert_attributes_into_widgets(self):
@@ -168,28 +81,46 @@ class MenuGuiBase(BaseGui):
             
         # Bind all the methods and annotations
         base_members = set(x[0] for x in iter_members(MenuGuiBase))        
+        
+        _hist: list[tuple[str, str, str]] = [] # for traceback
+        
         for name, attr in filter(lambda x: x[0] not in base_members, iter_members(cls)):
-            if isinstance(attr, type):
-                # Nested magic-menu
-                widget = attr()
-                setattr(self, name, widget)
-            
-            elif isinstance(attr, MagicField):
-                widget = self._create_widget_from_field(name, attr)
+            try:
+                if isinstance(attr, type):
+                    # Nested magic-menu
+                    widget = attr()
+                    setattr(self, name, widget)
                 
-            else:
-                # convert class method into instance method
-                if not hasattr(attr, "__magicclass_wrapped__"):
-                    widget = getattr(self, name, None)
+                elif isinstance(attr, MagicField):
+                    widget = self._create_widget_from_field(name, attr)
+                    
                 else:
-                    # If the method is redefined, the newer one should be used instead, while the
-                    # order of widgets should be follow the place of the older method.
-                    widget = attr.__magicclass_wrapped__.__get__(self)
+                    # convert class method into instance method
+                    if not hasattr(attr, "__magicclass_wrapped__"):
+                        widget = getattr(self, name, None)
+                    else:
+                        # If the method is redefined, the newer one should be used instead, while the
+                        # order of widgets should be follow the place of the older method.
+                        widget = attr.__magicclass_wrapped__.__get__(self)
+                
+                if name.startswith("_"):
+                    continue
+                if callable(widget) or isinstance(widget, (MenuGuiBase, Action, Separator, Container)):
+                    self.append(widget)
+                    _hist.append((name, str(type(attr)), type(widget).__name__))
             
-            if name.startswith("_"):
-                continue
-            if callable(widget) or isinstance(widget, (MenuGuiBase, Action, Separator, Container)):
-                self.append(widget)
+            except Exception as e:
+                hist_str = "\n\t".join(map(
+                    lambda x: f"{x[0]} {x[1]} -> {x[2]}",
+                    _hist
+                    )) + f"\n\t\t{name} ({type(attr).__name__}) <--- Error"
+                if not hist_str.startswith("\n\t"):
+                    hist_str += "\n\t"
+                if isinstance(e, InvalidMagicClassError):
+                    e.args = (f"\n{hist_str}\n{e}",)
+                    raise e
+                else:
+                    raise InvalidMagicClassError(f"{hist_str}\n\n{type(e).__name__}: {e}")
         
         return None
     
@@ -234,21 +165,38 @@ class ContextMenuGui(MenuGuiBase):
     """Magic class that will be converted into a context menu."""
     # TODO: Prevent more than one context menu
 
-def _value_widget_callback(mgui: MenuGuiBase, widget: ButtonWidget, name: str):
+def _value_widget_callback(mgui: MenuGuiBase, widget: ButtonWidget, name: str, getvalue: bool = True):
     def _set_value():
         if not widget.enabled:
             # If widget is read only, it means that value is set in script (not manually).
             # Thus this event should not be recorded as a macro.
             return None
-        value = widget.value
-        if isinstance(value, Exception):
+        
+        if isinstance(widget.value, Exception):
             return None
-        sub = Expr(head=Head.getattr, args=[Symbol(name), Symbol("value")]) # name.value
-        expr = Expr(head=Head.setattr, args=[symbol(mgui), sub, value]) # {x}.name.value = value
+        
+        mgui.changed.emit(mgui)
+        
+        if getvalue:
+            sub = Expr(head=Head.getattr, args=[Symbol(name), Symbol("value")]) # name.value
+        else:
+            sub = Expr(head=Head.value, args=[Symbol(name)])
+        
+        # Make an expression of
+        # >>> x.name.value = value
+        # or
+        # >>> x.name = value
+        expr = Expr(head=Head.assign, 
+                    args=[Expr(head=Head.getattr, 
+                               args=[symbol(mgui), sub]), 
+                          widget.value])
         
         last_expr = mgui._recorded_macro[-1]
-        if last_expr.head == expr.head and last_expr.args[1].args[0] == expr.args[1].args[0]:
+        if (last_expr.head == expr.head and 
+            last_expr.args[0].args[1].head == expr.args[0].args[1].head and
+            last_expr.args[0].args[1].args[0] == expr.args[0].args[1].args[0]):
             mgui._recorded_macro[-1] = expr
         else:
             mgui._recorded_macro.append(expr)
+        return None
     return _set_value
\ No newline at end of file
