@@ -6,10 +6,11 @@ These widgets are all compatible with the ``append`` method of Container widgets
 from magicgui.widgets import * # to avoid importing both magicgui.widgets and magicclass.widgets
 
 from .listwidget import ListWidget
-from .misc import Figure, ConsoleTextEdit, MacroEdit, CheckButton
+from .misc import Figure, ConsoleTextEdit, MacroEdit, CheckButton, show_messagebox
 from .separator import Separator
 from .sequence import ListEdit, TupleEdit
-from .utils import FrozenContainer
+from .threading import ProgressWidget, progress
+from .utils import FreeWidget
 
 class NotInstalled:
     def __init__(self, msg):
@@ -49,10 +50,13 @@ __all__ = ["ListWidget",
            "ConsoleTextEdit", 
            "MacroEdit",
            "CheckButton",
+           "show_messagebox",
            "Separator",
            "ListEdit",
            "TupleEdit",
-           "FrozenContainer",
+           "ProgressWidget",
+           "progress",
+           "FreeWidget",
            "QtConsole",
            "QtPlotCanvas",
            "QtImageCanvas",
