@@ -1,18 +1,19 @@
 from __future__ import annotations
 from typing import TYPE_CHECKING
+from enum import Enum
 from qtpy.QtGui import QFont, QTextOption, QGuiApplication
-from qtpy.QtWidgets import QMenuBar, QMenu, QAction
+from qtpy.QtWidgets import QMenuBar, QMenu, QAction, QMessageBox
 import matplotlib as mpl
 import matplotlib.pyplot as plt
 from matplotlib.backends.backend_qt5agg import FigureCanvas
 from magicgui.widgets import PushButton, TextEdit, FileEdit
 
-from .utils import FrozenContainer
+from .utils import FreeWidget
 
 if TYPE_CHECKING:
     from qtpy.QtWidgets import QWidget
 
-class Figure(FrozenContainer):
+class Figure(FreeWidget):
     """
     A matplotlib figure canvas.
     """    
@@ -20,8 +21,7 @@ class Figure(FrozenContainer):
                  nrows: int = 1,
                  ncols: int = 1,
                  figsize: tuple[int, int] = (4, 3),
-                 style = None,
-                 layout: str = "vertical", 
+                 style = None, 
                  **kwargs):
         backend = mpl.get_backend()
         try:
@@ -34,7 +34,7 @@ class Figure(FrozenContainer):
         finally:
             mpl.use(backend)
         
-        super().__init__(layout=layout, labels=False, **kwargs)
+        super().__init__(**kwargs)
         canvas = FigureCanvas(fig)
         self.set_widget(canvas)
         self.figure = fig
@@ -66,7 +66,7 @@ class ConsoleTextEdit(TextEdit):
         self.native.setFont(QFont("Consolas"))
         self.native.setWordWrapMode(QTextOption.NoWrap)
         
-class MacroEdit(FrozenContainer):
+class MacroEdit(FreeWidget):
     """
     A text edit embeded with a custom menu bar.
     """    
@@ -75,6 +75,7 @@ class MacroEdit(FrozenContainer):
         self.textedit = ConsoleTextEdit()
         self.set_widget(self.textedit.native)
         self.native: QWidget
+        self.native.setWindowTitle("Macro")
         
         self._menubar = QMenuBar(self.native)
         self.native.layout().setMenuBar(self._menubar)
@@ -141,3 +142,27 @@ class CheckButton(PushButton):
     def __init__(self, text: str | None = None, **kwargs):
         super().__init__(text=text, **kwargs)
         self.native.setCheckable(True)
+
+class MessageBoxMode(Enum):
+    ERROR = "error"
+    WARNING = "warn"
+    INFO = "info"
+    QUESTION = "question"
+    ABOUT = "about"
+
+QMESSAGE_MODES = {
+    MessageBoxMode.ERROR: QMessageBox.critical,
+    MessageBoxMode.WARNING: QMessageBox.warning,
+    MessageBoxMode.INFO: QMessageBox.information,
+    MessageBoxMode.QUESTION: QMessageBox.question,
+    MessageBoxMode.ABOUT: QMessageBox.about,
+}
+
+def show_messagebox(mode: str | MessageBoxMode = MessageBoxMode.INFO,
+                    title: str = None,
+                    text: str = None,
+                    parent=None
+                    ) -> bool:
+    show_dialog = QMESSAGE_MODES[MessageBoxMode(mode)]
+    result = show_dialog(parent, title, text)
+    return result in (QMessageBox.Ok, QMessageBox.Yes)
